@@ -25,7 +25,7 @@ from mautrix.util.async_db import Database
 
 from ..util import id_to_str
 
-fake_db = Database("") if TYPE_CHECKING else None
+fake_db = Database.create("") if TYPE_CHECKING else None
 
 
 @dataclass
